#include <string>
#include <sstream>
#include <fstream>
#include <vector>
#include <objbase.h>

// uses the same namespace as Form1
namespace odemxProjectCreator {

	using namespace std;

	public ref class ProjectWriter {
	public:

		// setters and getters
		void setProjectName( System::String^ name ) {
			projectName = name;
		}

		System::String^ getProjectName() {
			return projectName;
		}

		void setTemplateFile( System::String^ absoluteFilePath ) {
			templateFile = absoluteFilePath;
		}

		System::String^ getTemplateFile() {
			return templateFile;
		}

		void setOdemxLocation( System::String^ absolutePath ) {
			odemxLocation = absolutePath;
		}

		System::String^ getOdemxLocation() {
			return odemxLocation;
		}

		void writeProjectFile( System::String^ projectFilePath ) {

			// the VS project template needs at least three GUIDs
			vector<string>& odemxGUIDs = createGUIDs( 3 );

			/*
			// test GUID creation, display in a message window
			ostringstream GUIDstream;

			GUIDstream << "Created GUIDs:" << endl;

			vector<string>::iterator iter;
			for( iter = odemxGUIDs.begin(); iter != odemxGUIDs.end(); ++iter ) {
				GUIDstream << "  " << *iter << endl;
			}
			
			// System::String^ display = gcnew System::String( GUIDstream.str().c_str() );
			// System::Windows::Forms::MessageBox::Show( display );
			*/

			// open the project template file for reading
			ifstream inFile;

			// convert managed VC++ string into standard string
			string templateFilePath;
			MarshalString( templateFile, templateFilePath );

			// standard C++ file access
			inFile.open( templateFilePath.c_str() );
			
			if( !inFile ) {
				System::Windows::Forms::MessageBox::Show( "Error: could not open VS template file" );
				exit( -1 );
			}

			// delete the project file if it exists
			if ( System::IO::File::Exists( projectFilePath ) ) {
				System::IO::File::Delete( projectFilePath );
			}

			// stream writer for .NET style file acces
			System::IO::StreamWriter^ projectFileStream = gcnew System::IO::StreamWriter( projectFilePath, true, System::Text::Encoding::ASCII );

			string currentLine;
			// read file until EOF
			while( getline( inFile, currentLine, '\n' ) ) {

				// find odemx place holders
				string::size_type locStart = currentLine.find( "$(odemx" );

				if( locStart != string::npos ) {

					// VSProjectTemplate.xml contains the following place holders:
					// $(odemxProjectName)
					// $(odemxProjectGUID)
					// $(odemxHeaderGUID)
					// $(odemxSourceGUID)
					// $(odemxLocation)

					// get the length of the variable string to be replaced
					string::size_type length = currentLine.find( ')', locStart ) - locStart + 1;

					// compare the substring with the possible place holders
					// and replace their contents accordingly
					const string choice = currentLine.substr( locStart, length );

					if( choice == string( "$(odemxProjectName)" ) ) {
						string stdstrProjectName;
						MarshalString( projectName, stdstrProjectName );
						currentLine.replace( locStart, length, stdstrProjectName );
					}
					else if( choice == string( "$(odemxProjectGUID)" ) ) {
						currentLine.replace( locStart, length, odemxGUIDs[0] );
					}
					else if( choice == string( "$(odemxHeaderGUID)" ) ) {
						currentLine.replace( locStart, length, odemxGUIDs[1] );
					}
					else if( choice == string( "$(odemxSourceGUID)" ) ) {
						currentLine.replace( locStart, length, odemxGUIDs[2] );
					}
					else if( choice == string( "$(odemxLocation)" ) ) {
						string stdstrOdemxLocation;
						MarshalString( odemxLocation, stdstrOdemxLocation );
						currentLine.replace( locStart, length, stdstrOdemxLocation );
					}
					else {
						System::Windows::Forms::MessageBox::Show( "Error: unknown place holder in VS template file" );
						exit( -1 );
					}
				}

				// append (substituted) line
				projectFileStream -> WriteLine( (gcnew System::String( currentLine.c_str() )) );
			}

			// close template file and project file
			inFile.close();
			projectFileStream -> Close();
		}

	private:
		System::String^ projectName;
		System::String^ templateFile;
		System::String^ odemxLocation;

		vector<string> createGUIDs( unsigned short GUIDcount ) {

			// vector for holding globally unique identifiers as string
			vector<string> odemxGUIDs;

			for( unsigned short i = 0; i < GUIDcount; ++i ) {

				// declare variable of type GUID
				GUID theGUID;

				// fill the GUID struct
				CoCreateGuid( &theGUID );

				// convert struct to ANSI: unsigned char*
				unsigned char* cStringGUID = 0x00;
				UuidToStringA( &theGUID, &cStringGUID );

				// convert unsigned char* to string and store in vector
				ostringstream convert;
				convert << cStringGUID;
				odemxGUIDs.push_back( convert.str() );
			}

			if( odemxGUIDs.size() != GUIDcount ) {
				
				System::Windows::Forms::MessageBox::Show( "Error: could not create enough GUIDs to create project" );
				exit( -1 );
			}

			return odemxGUIDs;
		}

		void MarshalString ( System::String ^ s, std::string& os ) {
			using namespace System::Runtime::InteropServices;
			const char* chars = 
			(const char*)(Marshal::StringToHGlobalAnsi(s)).ToPointer();
			os = chars;
			Marshal::FreeHGlobal(System::IntPtr((void*)chars));
		}

	}; // class ProjectWriter

} // namespace odemxProjectCreator