//----------------------------------------------------------------------------
//	Copyright (C) 2002, 2004, 2007 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file HtmlGenerator.h

	\author Ronald Kluth

	\date created at 2007/11/01

	\brief Declaration of odemx::HtmlGenerator

	\sa HtmlGenerator.cpp

	<!-- [detailed description] -->

	\since 2.0
*/

#ifndef ODEMX_HTML_GENERATOR_INCLUDED
#define ODEMX_HTML_GENERATOR_INCLUDED

#include <iostream>
#include <ostream>
#include <fstream>
#include <string>
#include <odemx/base/Simulation.h>

namespace odemx {

	/** \class HtmlGenerator

		\ingroup display

		\author Ronald Kluth

		\brief HTML generator for XmlTrace

		\sa XmlTrace

		<!-- [\note {notes}]* -->

		<!-- [detailed description] -->

		<!-- [\warning {warnings}]* -->

		<!-- [\bug {bug description}]* -->

		<!-- [\test {testcase description}]* -->

		\since 2.0
	*/
	class HtmlGenerator {
	protected:

		/// Construction for DefaultSimulation
		HtmlGenerator();

		/// Construction for user-defined Simulation
		HtmlGenerator( Simulation* sim );

		/// produce HTML output from xml files
		void generateHtml( const std::string& fileName, const std::string& traceFilter = "" );

		// Implementation
	private:

		/**
			\name Html generation help functions

			Generation of an HTML file belonging to a trace is
			done in several steps, involving header construction
			with script and menu inclusion, and building the HTML
			file's content body.

			@{
		*/
		void htmlHead( const std::string& xmlFileName ); ///< build head with style, script
		void htmlBodyCaption( const std::string& filter ); ///< build page caption
		void htmlBodyNavigation(); ///< build page navigation
		void htmlBodyFoot(); ///< build page content and close
		//@}

	private:
		Simulation* env;
		std::ostream* htmlStream; ///< outstream for HTML text
	};

}


#endif /* ODEMX_HTML_GENERATOR_INCLUDED */
