//----------------------------------------------------------------------------
//	Copyright (C) 2002 - 2008 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file ProtocolSimulation.cpp

	\author Ronald Kluth

	\date created at 2007/04/04

	\brief Implementation of classes in ProtocolSimulation.h

	\sa ProtocolSimulation.h

	<!-- [detailed description] -->

	\since 2.0
*/

#include <odemx/protocol/ProtocolSimulation.h>
#include <odemx/protocol/TransmissionMedium.h>
#include <odemx/util/ErrorHandling.h>

#include <iostream>
#include <vector>

using namespace odemx;
using namespace std;

ProtocolSimulation::ProtocolSimulation( Label l, SimulationObserver* o ):
	Simulation( l, o ),
	medium( 0 ),
	layerManager()
	{}

ProtocolSimulation::ProtocolSimulation( Label l, SimTime startAt, SimulationObserver* o ):
	Simulation( l, startAt, o ),
	medium( 0 ),
	layerManager()
	{}

ProtocolSimulation::~ProtocolSimulation()
	{
		if( medium ) delete medium;
	}

NetTopology* ProtocolSimulation::getNetTopology()
{
	if ( !medium )
		error( this,  "ProtocolSimulation::getNetTopology(): NetTopology not available without TransmissionMedium" );

	return static_cast< NetTopology* >( medium );
}

TransmissionMedium* ProtocolSimulation::getTransmissionMedium()
{
	if ( !medium )
		error( this, "ProtocolSimulation::getTransmissionMedium(): TransmissionMedium not created, Simulation not initialized?" );

	return medium;
}

ProtocolLayerList& ProtocolSimulation::getLayerList()
{
	return layerManager.getLayerList();
}

ProtocolLayer* ProtocolSimulation::getLayerByLabel( Label l )
{
	return layerManager.getLayerByLabel( l );
}

void ProtocolSimulation::printLayers( bool withEntities /*=false*/ ) {
	std::cout << "Layers:" << std::endl;
	ProtocolLayerList::iterator iter;
	for( iter = getLayerList().begin(); iter != getLayerList().end(); ++iter )
	{
		std::cout << (*iter) -> getLabel() << std::endl;
		if ( withEntities )
			(*iter) -> printEntityList();
	}
}
