//----------------------------------------------------------------------------
//	Copyright (C) 2002, 2003, 2004 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file Queue.cpp

	\author Ralf Gerstenberger

	\date created at 2003/06/28

	\brief Implementation of Queue

	\sa Queue.h

	<!-- [detailed description] -->

	\since 1.0
*/

#ifdef _MSC_VER
#pragma warning (disable : 4786)
#endif

#include <odemx/synchronization/Queue.h>
#include <odemx/base/Process.h>

#include <string>
#include <vector>

using namespace std;
using namespace odemx;

Queue::Queue( Label l ):
	ProcessQueue(),
	DefLabeledObject( getDefaultSimulation(), l ),
	StatisticObject( getDefaultSimulation() ),
	queueStatistics( l + " statistics" )
	{
		// init statistics
		reset();
	}

Queue::Queue(Simulation* s, Label l) :
	ProcessQueue(),
	DefLabeledObject(s, l),
	StatisticObject(s),
	queueStatistics(s, (l + " statistics"))
	{
		assert(s!=0);

		// init statistic
		reset();
	}

void Queue::popQueue()
{
	ProcessQueue::popQueue();
	queueStatistics.update(getNow());
}

void Queue::remove(Process* p)
{
	ProcessQueue::remove(p);
	queueStatistics.update(getNow());
}

void Queue::inSort(Process* p,  bool fifo)
{
	ProcessQueue::inSort(p, fifo);
	queueStatistics.update(getNow());
}

void Queue::reset()
{
	StatisticObject::reset();
	queueStatistics.reset();
	queueStatistics.update(getNow());
}

void Queue::report(Report* r) {
	assert(r);

	static vector< string > labels;			static vector< Column::Type  > types;
	labels.push_back("Name");				types.push_back( Column::STRING);
	labels.push_back("Reset at");			types.push_back( Column::SIMTIME);
	labels.push_back("Min queue length");	types.push_back( Column::INTEGER);
	labels.push_back("Max queue length");	types.push_back( Column::INTEGER);
	labels.push_back("Now queue length");	types.push_back( Column::INTEGER);
	labels.push_back("Avg queue length");	types.push_back( Column::REAL);

	static utTableDef def( labels.size(), labels, types );

	Table* t=r->createTable("Queue Statistics", &def);
	assert(t);

	*t
	<< getLabel()
	<< getResetTime()
	<< getMin()
	<< getMax()
	<< getNow()
	<< getAVLength();
}

