//----------------------------------------------------------------------------
//	Copyright (C) 2002 - 2008 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file WaitCondition.h

	\author Ronald Kluth

	\date created at 2008/12/18

	\brief Declaration of odemx::WaitCondition

	\sa WaitCondition.cpp

	\since 2.1
*/

#ifndef ODEMX_WAITCONDITION_INCLUDED
#define ODEMX_WAITCONDITION_INCLUDED

#include <odemx/synchronization/Memory.h>
#include <odemx/util/LabeledObject.h>

namespace odemx
{

/** \class WaitCondition

	\ingroup synch

	\author Ronald Kluth

	\brief %WaitCondition is a memory class that allows processes to wait for
	an arbitrary condition. 

	\sa Memory, Process, Timer, PortHead, PortTail

	%Memory is an abstract base class for all user defined structures in a model
	that need to 
	
	WaitCondition can store a list of suspended processes or other schedulable
	objects in order to wake them when a specified condition becomes true. The
	suspended processes can be reactivated by calling \c notify(). 

	\since 2.1
*/
class WaitCondition:
	public odemx::Memory,
	public odemx::DefLabeledObject
{
public:
	/// Constructor
	WaitCondition( Simulation* sim, Label l );

	/// override of pure virtual function from Memory, always returns false 
	virtual bool available(); 

	/**
	 * @brief The function to call when the arbitrary condition is fulfilled
	 */
	void notify();
};


/** \interface WaitConditionObserver

	\author Ronald Kluth

	\brief Observer for WaitCondition-specific calls.

	\sa Memory, Timer, PortHead, PortTail

	\since 2.1
*/
class WaitConditionObserver:
	public odemx::MemoryObserver
{
public:
	virtual ~WaitConditionObserver() {}
	
	// observation of construction/destruction in MemoryObserver
	
	virtual void onNotify( WaitCondition* sender ) {} ///< notify all processes
};

} // namespace odemx

#endif
