using namespace odemx;
using namespace std;

#include <odemx/util/TypedObject.h>

/********************* Port Template Implementation ***************************/

template < typename ElementType >
PortT< ElementType >::PortT( Label l, unsigned int max,
							 PortHeadT< ElementType >* ph,
							 PortTailT< ElementType >* pt ):
	DefLabeledObject( getDefaultSimulation(), l ),
	StatisticObject( getDefaultSimulation() ),
	maxCapacity( max ),
	portHead( ph ),
	portTail( pt ),
	env( getDefaultSimulation() ),
	portAccum( getDefaultSimulation() ),
	putCalls( 0 ),
	getCalls( 0 ),
	ungetCalls( 0 ),
	headAlerts( 0 ),
	tailAlerts( 0 )
	{
		elementTypeName = typeToString( typeid(ElementType) );
	}

template < typename ElementType >
PortT< ElementType >::PortT( Simulation* sim, Label l, unsigned int max,
							 PortHeadT< ElementType >* ph,
							 PortTailT< ElementType >* pt ):
	DefLabeledObject( sim, l ),
	StatisticObject( sim ),
	maxCapacity( max ),
	portHead( ph ),
	portTail( pt ),
	env( sim ),
	portAccum( sim ),
	putCalls( 0 ),
	getCalls( 0 ),
	ungetCalls( 0 ),
	headAlerts( 0 ),
	tailAlerts( 0 )
	{
		elementTypeName = typeToString( typeid(ElementType) );
	}

template < typename ElementType >
PortT< ElementType >::~PortT()
	{
		if ( !elementList.empty() ) {
			// warning: port still contains elements
			warning( this, "Port::~Port(): Port not empty, beware of memory leak" );
		}
	}

template < typename ElementType >
void PortT< ElementType >::report( Report* r ) {
	assert(r);

	static vector< string > labels;			static vector< Column::Type  > types;
	labels.push_back("Name");				types.push_back( Column::STRING);
	labels.push_back("Element type");		types.push_back( Column::STRING);
	labels.push_back("Buffer limit");		types.push_back( Column::INTEGER);
	labels.push_back("Reset at");			types.push_back( Column::SIMTIME);
	labels.push_back("Put calls");			types.push_back( Column::INTEGER);
	labels.push_back("Get calls");			types.push_back( Column::INTEGER);
	labels.push_back("Unget calls");		types.push_back( Column::INTEGER);
	labels.push_back("Min buffer count");	types.push_back( Column::INTEGER);
	labels.push_back("Max buffer count");	types.push_back( Column::INTEGER);
	labels.push_back("Now buffer count");	types.push_back( Column::INTEGER);
	labels.push_back("Avg buffer count");	types.push_back( Column::REAL);
	labels.push_back("Head alerts");		types.push_back( Column::INTEGER);
	labels.push_back("Tail alerts");		types.push_back( Column::INTEGER);

	static utTableDef def( labels.size(), labels, types );

	Table* t = r -> createTable( "Port Statistics", &def );
	assert(t);

	*t
	<< getLabel()
	<< elementTypeName
	<< maxCapacity
	<< getResetTime()
	<< putCalls
	<< getCalls
	<< ungetCalls
	<< getMinCount()
	<< getMaxCount()
	<< getNowCount()
	<< getAvgCount()
	<< headAlerts
	<< tailAlerts;
}

template < typename ElementType >
void PortT< ElementType >::reset() {
	putCalls = 0;
	getCalls = 0;
	ungetCalls = 0;
	headAlerts = 0;
	tailAlerts = 0;
	StatisticObject::reset();

	portAccum.reset();
	portAccum.update(getNowCount());
}

