/**
 * @file TestDisplay.h
 * @date Aug 15, 2008
 * @author Ronald Kluth
 *
 * @brief Contains all helper classes required for testing ODEMx module \b %Display.
 */

#ifndef TESTDISPLAY_H_
#define TESTDISPLAY_H_

#include <UnitTest++.h>
#include "../Main/Globals.h"

#include <odemx/synchronization/Timer.h> // for filter test
#include <odemx/display/HtmlGenerator.h>
#include <odemx/display/HtmlReport.h>
#include <odemx/display/HtmlTrace.h>
#include <odemx/display/TimeFormat.h>
#include <odemx/display/TraceFilter.h>
#include <odemx/display/XmlReport.h>
#include <odemx/display/XmlTrace.h>

/**
 * @namespace Display
 *
 * @brief Contains helpers and tests for module ODEMx %Display
 *
 * The UnitTest++ macro SUITE( Display ) declares namespace Display.
 * Consequently, all helpers for this test suite should be placed in
 * the same namespace.
 */
namespace Display {

	using namespace odemx;
	using namespace std;

	/********************* TRACE FILTER ***************************************/

	/**
	 * @class TraceFilterTest
	 * @brief Subclass of TraceFilter to check some protected functions
	 */
	class TraceFilterTest:
		public TraceFilter
	{
	public:
		/**
		 * @name Protected functions made public for testing
		 * @{
		 */
		bool pass(const MarkType* m, const TraceProducer* sender)
		{
			return TraceFilter::pass( m, sender );
		}
		bool hasPassed() const
		{
			return TraceFilter::hasPassed();
		}
		bool foundMatch()
		{
			return TraceFilter::foundMatch();
		}
		//@}
	};

	/************************** TIME FORMAT ***********************************/

	/**
	 * @class TimeFormatTest
	 * @brief Helper class for testing Time::Format, implements a pure virtual
	 * function
	 */
	class TimeFormatTest:
		public Time::Format
	{
	public:
		/// Default construction
		TimeFormatTest(): Time::Format() {}

		/// Construction with Time::Base
		TimeFormatTest( Time::Base tb ): Time::Format( tb ) {}

		/// implementation of pure virtual function, returns strings for testing
		virtual const std::string makeString() const
		{
			std::ostringstream timeStream;
			timeStream
			<< "time: "
			<< year << "/" << month << "/" << day << "-"
			<< hour << ':' << min << ':' << sec;
			return timeStream.str();
		}
	};

} // namespace Display

#endif /* TESTDISPLAY_H_ */
