/**
 * @file TestContinuousDist.cpp
 * @date Aug 9, 2008
 * @author Ronald Kluth
 *
 * @brief Tests for ODEMx class ContinuousDist
 */

#include "TestRandom.h"
#include "../TestBase/TestBase.h"

/// @cond DOXYGEN_SKIP
SUITE( Random )
{
/// @endcond

	/**
	 * @struct ContinuousDistFixture
	 * @brief Helper struct providing set-up/tear-down of ContinuousDist tests
	 *
	 * @copydetails EventFixture
	 */
	struct ContinuousDistFixture
	{
		Base::SimulationTest sim;
		ContinuousDistTest dist;

		ContinuousDistFixture()
		:	sim( "ContinuousDistTestSim" ),
			dist( &sim, "ContinuousDistTest" )
			{}
	};

	/**
	 * @test odemx::ContinuousDist construction
	 *
	 * Expected function call effects:
	 * @li dist context (simulation) is set correctly
	 * @li label is set correctly
	 */
	TEST_FIXTURE( ContinuousDistFixture, ConstructionDestruction )
	{
		Label l = "ContinuousDistTestDefSimConstruction";
		ContinuousDistTest dist1( l );
		CHECK_EQUAL( l, dist1.getLabel() );
		CHECK_EQUAL( getDefaultSimulation(), dist1.getStatisticManager() );

		l = "ContinuousDistTestUserSimConstruction";
		ContinuousDistTest dist2( &sim, l );
		CHECK_EQUAL( l, dist2.getLabel() );
		CHECK_EQUAL( &sim, dist2.getStatisticManager() );
	}

	/**
	 * @test odemx::ContinuousDist uses
	 *
	 * Expected function call effects:
	 * @li check correct return value of usage counter
	 */
	TEST_FIXTURE( ContinuousDistFixture, Uses )
	{
		dist.sample();
		CHECK_EQUAL( 1, dist.getUses() );
	}

	/**
	 * @test odemx::ContinuousDist::reset()
	 *
	 * Expected function call effects:
	 * @li usage counter is 0
	 */
	TEST_FIXTURE( ContinuousDistFixture, Reset )
	{
		dist.sample();
		CHECK_EQUAL( 1, dist.getUses() );

		dist.reset();
		CHECK_EQUAL( 0, dist.getUses() );
	}

/// @cond DOXYGEN_SKIP
}
/// @endcond
