/**
 * @file TestDiscreteDist.cpp
 * @date Aug 9, 2008
 * @author Ronald Kluth
 *
 * @brief Tests for ODEMx class DiscreteDist
 */

#include "TestRandom.h"
#include "../TestBase/TestBase.h"

/// @cond DOXYGEN_SKIP
SUITE( Random )
{
/// @endcond

	/**
	 * @struct DiscreteDistFixture
	 * @brief Helper struct providing set-up/tear-down of DiscreteDist tests
	 *
	 * @copydetails EventFixture
	 */
	struct DiscreteDistFixture
	{
		Base::SimulationTest sim;
		DiscreteDistTest dist;

		DiscreteDistFixture()
		:	sim( "DiscreteDistTestSim" ),
			dist( &sim, "DiscreteDistTest" )
			{}
	};

	/**
	 * @test odemx::DiscreteDist construction
	 *
	 * Expected function call effects:
	 * @li dist context (simulation) is set correctly
	 * @li label is set correctly
	 */
	TEST_FIXTURE( DiscreteDistFixture, ConstructionDestruction )
	{
		Label l = "DiscreteDistTestDefSimConstruction";
		DiscreteDistTest dist1( l );
		CHECK_EQUAL( l, dist1.getLabel() );
		CHECK_EQUAL( getDefaultSimulation(), dist1.getStatisticManager() );

		l = "DiscreteDistTestUserSimConstruction";
		DiscreteDistTest dist2( &sim, l );
		CHECK_EQUAL( l, dist2.getLabel() );
		CHECK_EQUAL( &sim, dist2.getStatisticManager() );
	}

	/**
	 * @test odemx::DiscreteDist uses
	 *
	 * Expected function call effects:
	 * @li check correct return value of usage counter
	 */
	TEST_FIXTURE( DiscreteDistFixture, Uses )
	{
		dist.sample();
		CHECK_EQUAL( 1, dist.getUses() );
	}

	/**
	 * @test odemx::DiscreteDist::reset()
	 *
	 * Expected function call effects:
	 * @li usage counter is 0
	 */
	TEST_FIXTURE( DiscreteDistFixture, Reset )
	{
		dist.sample();
		CHECK_EQUAL( 1, dist.getUses() );

		dist.reset();
		CHECK_EQUAL( 0, dist.getUses() );
	}

/// @cond DOXYGEN_SKIP
}
/// @endcond
