/**
 * @file TestRandom.h
 * @date Aug 8, 2008
 * @author Ronald Kluth
 *
 * @brief Contains all helper classes required for testing ODEMx module \b %Random.
 */

#ifndef TESTRANDOM_H_
#define TESTRANDOM_H_

#include <UnitTest++.h>
#include "../Main/Globals.h"

#include <odemx/random/Dist.h>
#include <odemx/random/DiscreteDist.h>
#include <odemx/random/ContinuousDist.h>

/**
 * @namespace Random
 *
 * @brief Contains helpers and tests for module ODEMx %Random
 *
 * The UnitTest++ macro SUITE( Random ) declares namespace
 * Random. Consequently, all helpers for this test suite should
 * be placed in the same namespace.
 */

namespace Random {

	using namespace odemx;
	using namespace std;

	/***************************** DIST ***************************************/

	/**
	 * @class DistTest
	 * @brief Subclass for making Dist methods public and implementing pure
	 * virtual functions
	 */
	class DistTest:
		public Dist
	{
	public:
		/// Constructor
		DistTest( DistContext* dc, Label l )
		:	Dist( dc, l )
			{}

		/// Required implementation of pure virtual function
		virtual void report( Report* r ) {}

		/// Makes protected function public for testing
		double getSample()
		{
			return Dist::getSample();
		}
	};

	/*************************** DISCRETE DIST ********************************/

	/**
	 * @class DiscreteDistTest
	 * @brief Subclass that implements required pure virtual functions
	 */
	class DiscreteDistTest:
		public DiscreteDist
	{
	public:
		/// Constructor for DefaultSimulation
		DiscreteDistTest( Label l )
		:	DiscreteDist( l )
			{}
		/// Constructor for user-defined Simulation
		DiscreteDistTest( DistContext* dc, Label l )
		:	DiscreteDist( dc, l )
			{}
		/// Required implementation of pure virtual function
		virtual void report( Report* r ) {}
		/// Required implementation of pure virtual function
		virtual int sample() { ++uses; return 1; }
	};

	/************************* CONTINUOUS DIST ********************************/

	/**
	 * @class ContinuousDistTest
	 * @brief Subclass that implements required pure virtual functions
	 */
	class ContinuousDistTest:
		public ContinuousDist
	{
	public:
		/// Constructor for DefaultSimulation
		ContinuousDistTest( Label l )
		:	ContinuousDist( l )
			{}
		/// Constructor for user-defined Simulation
		ContinuousDistTest( DistContext* dc, Label l )
		:	ContinuousDist( dc, l )
			{}
		/// Required implementation of pure virtual function
		virtual void report( Report* r ) {}
		/// Required implementation of pure virtual function
		virtual double sample() { ++uses; return 1; }
	};

}

#endif /* TESTRANDOM_H_ */
