/**
 * @file TestStatistics.h
 * @date Aug 5, 2008
 * @author Ronald Kluth
 *
 * @brief Contains all helper classes required for testing ODEMx module \b %Statistics.
 */

#ifndef TESTSTATISTICS_H_
#define TESTSTATISTICS_H_

#include <UnitTest++.h>
#include "../Main/Globals.h"

#include <odemx/statistics/Statistics.h>
#include <memory>

/**
 * @namespace Statistics
 *
 * @brief Contains helpers and tests for module ODEMx %Statistics
 *
 * The UnitTest++ macro SUITE( Statistics ) declares namespace
 * Statistics. Consequently, all helpers for this test suite should
 * be placed in the same namespace.
 */

namespace Statistics {

	using namespace odemx;
	using namespace std;

	/***************************** Tab ****************************************/

	/**
	 * @class TabTest
	 * @brief Subclass of Tab that makes protected function public for testing
	 */
	class TabTest:
		public Tab
	{
	public:
		/// Constructor for DefaultSimulation
		TabTest( Label l )
		:	Tab( l )
			{}
		/// Constructor for user-defined Simulation
		TabTest( Simulation* s, Label l )
		:	Tab( s, l )
			{}
		/** name Required re-implementations of protected functions
		 * @{
		 */
		void update() { Tab::update(); }
		virtual void reset() { Tab::reset(); }
		unsigned int getObs() const { return Tab::getObs();}
		//@}
		/// required implementation of pure virtual function
		virtual void report( Report* ) {}
	};

}

#endif
