/**
 * @file TestTypedObject.cpp
 * @date Jul 7, 2008
 * @author Ronald Kluth
 *
 * @brief Test for ODEMx class TypedObject
 */

#include "TestUtil.h"

/// @cond DOXYGEN_SKIP
SUITE( Util )
{
/// @endcond

	/**
	 * @test odemx::TypedObject::getType()
	 *
	 * There is a difference between calling typeid() on a polymorphic
	 * pointer and calling getType(). While typeid() will only say
	 * that it is a pointer type, getType() will return the type_info of
	 * the actual polymorphic object.
	 *
	 * Expected function call effects on a polymorphic pointer:
	 * @li the returned type is that of the actual object
	 */
	TEST( TypedObjectGetType )
	{
		Simulation* sim = getDefaultSimulation();

		CHECK_EQUAL( typeid( sim ).name(), typeid( Simulation* ).name() );
		CHECK_EQUAL( sim -> getType().name(), typeid( DefaultSimulation& ).name() );
	}

	/**
	 * @test odemx::typeToString(type_info&)
	 *
	 * This test examines the string returned by calling \c
	 * typeToString( typeid( odemx::Simulation * ) ).
	 *
	 * Expected function call effects:
	 * @li namespace "odemx" is found in the string
	 * @li the scope operator "::" is found
	 * @li class name "Simulation" is found
	 * @li the "*" operator is found
	 * @li the substrings are in the correct order
	 */
	TEST( TypeToString )
	{
		size_t odemx, scope, sim, ast;
		string typeName = typeToString( typeid( odemx::Simulation * ) );
		CHECK( (odemx = typeName.find( "odemx" )) != string::npos );
		CHECK( (scope = typeName.find( "::" )) != string::npos );
		CHECK( (sim = typeName.find( "Simulation" )) != string::npos );
		CHECK( (ast = typeName.find( '*' )) != string::npos );
		CHECK( odemx < scope && scope < sim && sim < ast );
	}

/// @cond DOXYGEN_SKIP
}
/// @endcond

