//----------------------------------------------------------------------------
//	Copyright (C) 2002, 2004 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file LabeledObject.cpp

	\author Ralf Gerstenberger
	<!-- [\author <author>]* -->

	\date created at 2002/08/17

	\brief Implementation of classes in LabeledObject.h

	\sa LabeledObject.h

	<!-- [detailed description] -->



	\since 1.0
*/

#include <odemx/util/LabeledObject.h>

#include <sstream>

using namespace odemx;
using namespace std;


// unsigned long LabelScope::labeledWithinScope = 0;


LabelScope::LabelScope( const string& defLabel /*="NoName"*/, char defSpace /*='_'*/ ):
	def(defLabel),
	spc(defSpace)
	{}


LabeledObject::~LabeledObject()
	{}


const string LabelScope::createLabel( const string& label )
{
	string l( label );

	if ( l.empty() )
		l = def;

	unsigned int number = lookup[l]++;

	if ( number > 0 )
	{
		ostringstream stream;
		stream << l << spc << number;
		return stream.str();
	}

	return l;
}

DefLabeledObject::DefLabeledObject():
	l( "???" )
	{}

DefLabeledObject::DefLabeledObject( LabelScope* scope, const string& label )
	{
		if ( scope != 0 )
			l = scope -> createLabel( label );
		else
			l = label;
	}

Label DefLabeledObject::getLabel() const {
	return l;
}

void DefLabeledObject::setLabel( LabelScope* scope, const string& label ) {

	if ( scope!=0 )
		l = scope -> createLabel( label );
	else
		l = label;
}

