//----------------------------------------------------------------------------
//	Copyright (C) 2002, 2004, 2007 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file MarkType.cpp

	\author Ralf Gerstenberger
	<!-- [\author <author>]* -->

	\date created at 2002/03/19

	\brief Implementation of MarkType, Tag

	\sa MarkType.h

	<!-- [detailed description] -->

	<!-- [\since {since when in project}] -->
*/

#include <odemx/util/TypedObject.h>
#include <odemx/util/MarkType.h>
#include <iostream>

using namespace odemx;
using namespace std;

MarkType::MarkType(const string& n, MarkTypeId i, const type_info& s):
	name(n),
	id(i),
	scope(s)
	{
		getList().push_back( this );
	}

MarkType::~MarkType()
	{
		// make sure the list cannot access this mark anymore
		// since it stores pointers, that would lead to a segmentation fault
		getList().remove( this );
	}

const string MarkType::getName() const {
	return name;
}

MarkTypeId MarkType::getId() const {
	return id;
}

const type_info& MarkType::getScope() const {
	return scope;
}

// static member functions
const MarkTypeId MarkType::getNewBaseMarkId( const type_info& newType ) {

	static unsigned long baseMarkCounter = 1;

	const MarkTypeId newBaseMarkId = 100 * baseMarkCounter++;

	getScopeIdMap()[ typeToString( newType ) ] = newBaseMarkId;

	return newBaseMarkId;
}

MarkTypeList& MarkType::getList() {

	static MarkTypeList allMarks;
	return allMarks;
}

MarkScopeIdMap& MarkType::getScopeIdMap() {

	static MarkScopeIdMap scopeIds;
	return scopeIds;
}

void MarkType::printAll() {

	MarkTypeList::const_iterator i;

	for ( i = getList().begin(); i != getList().end(); ++i )
	{
		cout << typeToString( (*i) -> getScope() ) << " | ";
		cout << (*i) -> getId() << " | ";
		cout << (*i) -> getName() << endl;
	}
}

void MarkType::printScopeIds() {

	MarkScopeIdMap::const_iterator i;

	for ( i = getScopeIdMap().begin(); i != getScopeIdMap().end(); ++i )
	{
		cout << (*i).first << " : ";
		cout << (*i).second << endl;
	}
}

Tag::Tag(const string& n, TagId i):
	name(n),
	id(i)
	{}

const string Tag::getName() const {
	return name;
}

TagId Tag::getId() const {
	return id;
}

// static member initialization
const TagId Tag::NOID = 0;
