//----------------------------------------------------------------------------
//	Copyright (C) 2002 - 2008 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file Report.cpp

	\author Ralf Gerstenberger

	\date created at 2002/06/21

	\brief Implementation of report mechanism

	\sa Report.h

	<!-- [detailed description] -->

	\since 1.0
*/

#ifdef _MSC_VER
#pragma warning(disable : 4786)
#pragma warning(disable : 4530)
#endif

#include <odemx/util/Report.h>
#include <odemx/util/ErrorHandling.h>

#include <cassert>
#include <cstring>
#include <algorithm>

using namespace odemx;
using namespace std;


Report::~Report()
{
	std::vector<Table*>::iterator i;

	for (i=ts.begin(); i!=ts.end(); ++i)
	{
		delete *i;
	}
}


Table* Report::createTable(const string& name, TableDefinition* def) {
	Table* t = findTable(name, def);

	if ( ! t ) {
		t = new Table( name, def );
		ts.push_back(t);
	}

	return t;
}


Table* Report::findTable(const string& name, TableDefinition* def) {

	if ( def==0 || name.empty() )
		return 0;

	// compare this report's table definitions with the given one;
	// if their definitions are the same, the labels are used to find it
	std::vector<Table*>::const_iterator i;
	for (i=ts.begin(); i!=ts.end(); ++i)
	{
		if ( ! (*((*i)->def)==*def) )
			continue;

		if ( (*i)->getLabel() == name )
			return *i;
	}

	return 0;
}

void Report::generateReport() {
	std::list< ReportProducer* >::iterator i;

	// call all report producers to fill their tables
	for ( i = rs.begin(); i != rs.end(); ++i )
		(*i)->report(this);

	processTables();
}

utTableDef::utTableDef( size_t size,
						const vector< string >& labels,
						const vector< Column::Type >& types):
	sz(size),
	columnLabels(labels),
	columnTypes(types)
	{}

size_t utTableDef::getNumberOfColumns() const {
	return sz;
}

const string utTableDef::getLabelOfColumn(size_t i) const {

	return (i < sz)  ?  columnLabels[i] : 0;
}

Column::Type utTableDef::getTypeOfColumn(size_t i) const {

	return (i < sz)  ?  columnTypes[i] : Column::INVALID;
}

