//----------------------------------------------------------------------------
//	Copyright (C) 2002 - 2008 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file TypedObject.cpp

	\author Ronald Kluth

	\date created at 2007/05/10

	\brief implementation of type information

	\sa TypedObject.h

	<!-- [detailed description] -->

	\since 2.0
*/

#include <odemx/util/TypedObject.h>

// typeid().name returns a mangled type name on GCC
// needs to be treated differently than other compilers
// the following includes the demangling library header
#if defined(__GNUC__) /* GNU Compiler */

#include <cxxabi.h>

#endif

#include <stdlib.h>

using namespace std;
using namespace odemx;


TypedObject::~TypedObject()
	{}

const std::type_info& TypedObject::getType() const {
	return typeid(*this);
}

const std::string odemx::typeToString( const std::type_info& type ) {

#if defined(__GNUC__) /* GNU Compiler */

	int demangleStatus = -1;

	char* cString = abi::__cxa_demangle(type.name(), 0, 0, &demangleStatus);

	string typeString( cString );

	free( cString );

#else // other compilers should give a demangled name

	string typeString( type.name() );

#endif

	return typeString;
}
