//----------------------------------------------------------------------------
//	Copyright (C) 2002 - 2008 Humboldt-Universitaet zu Berlin
//
//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Lesser General Public
//	License as published by the Free Software Foundation; either
//	version 2.1 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Lesser General Public License for more details.
//
//	You should have received a copy of the GNU Lesser General Public
//	License along with this library; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------
/**	\file TypedObject.h

	\author Ralf Gerstenberger

	\date created at 2002/03/10

	\brief type information

	\sa TypedObject.cpp

	<!-- [detailed description] -->

	\since 1.0
*/

#ifndef ODEMX_TYPEDOBJECT_INCLUDED
#define ODEMX_TYPEDOBJECT_INCLUDED

#include <typeinfo>
#include <string>

namespace odemx {
	/** \class TypedObject

		\ingroup util

		\author Ralf Gerstenberger
		<!-- [\author <author>]* -->

		\brief Declaration of odemx::TypedObject and odemx::typeToString()

		<!-- [\note {notes}]* -->

		\sa odemx::typeToString()

		TypedObject provides c++ style type information
		about an object. TypedObject requires active RTTI
		support.

		<!-- [\warning {warnings}]* -->

		<!-- [\bug {bug description}]* -->

		<!-- [\test {testcase description}]* -->

		\since 1.0
	*/
	class TypedObject {
	public:
		virtual ~TypedObject();
		/**
			\brief get type information
		*/
		virtual const std::type_info& getType() const;
	};

	/**
	 * @brief Get a string representation of the type name
	 * @param type reference to a type_info object returned by typeid()
	 * @return the type name as string
	 *
	 * The function type_info::name() returns a mangled type name on GCC
	 * and thus needs to be treated differently than on other compilers.
	 * This function applies demangling to retrieve a human-readable
	 * string representation of type names.
	 */
	extern const std::string typeToString( const std::type_info& type );
}

#endif

